/* -*- c++ -*- */
/*
 * Copyright 2004,2010,2012 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

// WARNING: this file is machine generated. Edits will be overwritten

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "encoder_ss_impl.h"
#include <gnuradio/io_signature.h>
#include <iostream>

namespace gr {
  namespace trellis {

    encoder_ss::sptr
    encoder_ss::make(const fsm &FSM, int ST)
    {
      return gnuradio::get_initial_sptr
	(new encoder_ss_impl(FSM,ST,0,false));
    }

    encoder_ss::sptr
    encoder_ss::make(const fsm &FSM, int ST, int K)
    {
      return gnuradio::get_initial_sptr
	(new encoder_ss_impl(FSM,ST,K,true));
    }

    encoder_ss_impl::encoder_ss_impl(const fsm &FSM, int ST, int K, bool B)
    : sync_block("encoder_ss",
		    io_signature::make(1, 1, sizeof(short)),
		    io_signature::make(1, 1, sizeof(short))),
      d_FSM(FSM),
      d_ST(ST),
      d_K(K),
      d_B(B)      
    {
    }

    void encoder_ss_impl::set_FSM(const fsm &FSM) 
    { 
      gr::thread::scoped_lock guard(d_setlock);
      d_FSM = FSM; 
    }

    void encoder_ss_impl::set_ST(int ST) 
    { 
      gr::thread::scoped_lock guard(d_setlock);
      d_ST = ST; 
    }

    void encoder_ss_impl::set_K(int K) 
    { 
      gr::thread::scoped_lock guard(d_setlock);
      d_K = K; 
    }

    encoder_ss_impl::~encoder_ss_impl()
    {
    }

    int
    encoder_ss_impl::work(int noutput_items,
		      gr_vector_const_void_star &input_items,
		      gr_vector_void_star &output_items)
    {
      gr::thread::scoped_lock guard(d_setlock);
      int ST_tmp = 0;

      if (d_B){ // blockwise operation
        int nblocks = noutput_items /d_K;
	const short *in = (const short*)input_items[0];
	short *out = (short *) output_items[0];
        for(int n = 0; n < nblocks; n++) {
          ST_tmp = d_ST;
          for(int i = 0; i < d_K; i++) {
            out[n*d_K+i] = (short)d_FSM.OS()[ST_tmp*d_FSM.I()+in[n*d_K+i]];
            ST_tmp = (int)d_FSM.NS()[ST_tmp*d_FSM.I()+in[n*d_K+i]];
          }
        }
        return nblocks*d_K;
      } // end blockwise operation
      else{ // streaming operation
        const short *in = (const short*)input_items[0];
        short *out = (short *) output_items[0];
        ST_tmp = d_ST;
        for(int i = 0; i < noutput_items; i++) {
          out[i] = (short)d_FSM.OS()[ST_tmp*d_FSM.I()+in[i]]; 
          ST_tmp = (int)d_FSM.NS()[ST_tmp*d_FSM.I()+in[i]];
        }
        d_ST = ST_tmp;
        return noutput_items;
      } // end streaming operation  
   }

  } /* namespace trellis */
} /* namespace gr */
